#define ORBITER_MODULE
#include <stdio.h>
#include <math.h>
#include <osdk\orbitersdk.h>
#include <osdk\orbitersoundsdk35.h>

//Mesh Offsets
const VECTOR3 OFS_S1         = {0,0,0};
const VECTOR3 OFS_INTERSTAGE = {0,0,23.5};
const VECTOR3 OFS_S2         = {0,0,36.8};
const VECTOR3 OFS_PAYLOAD    = {0,0,62.8};
const VECTOR3 OFS_FAIR1      = {0,0,0};
const VECTOR3 OFS_FAIR2      = {0,0,0};

//Timings
const double skirtSepInterval = 10.0;
const double fairingInterval = 15.0;
const double stage2DeorbitInterval = 90;

const UINT MESH_UNDEFINED = (UINT)-1;

class saturnv21:public VESSEL2{
	
public:
	//Constructor and Destructor
	saturnv21(OBJHANDLE obj, int fmodel);
	~saturnv21(){}
	
	//Configuration depending on the actual stage
	void setLaunchConfiguration(void);
	void setFlyingConfiguration(void);
	void setStage2Configuration(void);

	//visual objects modifications after staging
	void addPayloadVisual(const VECTOR3 &ofs);
	void addStage1Visual(const VECTOR3 &ofs);
	void addStage2Visual(const VECTOR3 &ofs);
	
	//logical staging operations
	void discardStage1(void);
	void skirtSep(void);
	void fair(void);
	void releasePayload(void);
	
	// Overridden callback functions
	void clbkSetClassCaps(FILEHANDLE cfg);
	void clbkSetStateEx (const void *status);
	void clbkLoadStateEx (FILEHANDLE scn, void *vs);
	void clbkSaveState (FILEHANDLE scn);
//	void clbkFocusChanged (bool getfocus, OBJHANDLE hNewVessel, OBJHANDLE hOldVessel);
	void clbkPostStep (double simt, double simdt, double mjd);
	int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
//	void clbkVisualCreated (VISHANDLE vis, int refcount);
//	void clbkVisualDestroyed (VISHANDLE vis, int refcount);

private:
	void CreateStage2RCS(void);
	int status;
	bool fairing;
	double payloadWeight;
	double launchHDG;
	double desiredApogee;
	double desiredPerigee;
	ATTACHMENTHANDLE payload_attach;
	MESHHANDLE stage1Mesh,interstage, stage2Mesh,fairingMesh[2];
	PROPELLANT_HANDLE ph_1, ph_2;
	THRUSTER_HANDLE th_f1[5], th_j2[5];
	THGROUP_HANDLE thg_main, thg_rcs;
	UINT stage1, intstg, stage2, fairing1, fairing2;
	double sepCountdown;
	bool goForSkirtSep;
	bool goForFairing;
	bool waitSafeDistance;
	bool goForDeorbit;
};



DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int fmodel)
{
	return new saturnv21(hvessel, fmodel);
}

DLLCLBK void ovcExit(VESSEL *vessel){
	if(vessel) delete (saturnv21*)vessel;
}
